#import "ViewController.h"

#define gameIsRunning 1
#define gameIsPaused 0

@implementation ViewController
@synthesize ball, player1, player2, gameMode, ballSpeedX, ballSpeedY;
@synthesize scorePlayer1, scorePl1;
@synthesize startScreen;
@synthesize player1RichtungX, player2RichtungX, motionManager;
@synthesize scorePlayer2, scorePl2;

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Release any cached data, images, etc that aren't in use.
}

#pragma mark - View lifecycle

- (void) viewDidLoad {
  [super viewDidLoad];
  
  [self.view setMultipleTouchEnabled:YES];
  
  // Statusleiste entfernen und Hintergrundbeleuchtung ein
  [[UIApplication sharedApplication] setStatusBarHidden:YES];  
  [[UIApplication sharedApplication] setIdleTimerDisabled:YES];
  
  // Ballgeschwindigkeit festlegen
  ballSpeedX = arc4random()%10-5; // -5 bis +5
  ballSpeedY = 3;
  
  gameMode = gameIsPaused;
  
  scorePl1=0;
  scorePl2=0;
  
  // Bewegungssteuerung nutzen
  useMotionManager = false;

  // Gyroskopabfrage initialisieren
  motionManager = [[CMMotionManager alloc] init];
  motionManager.accelerometerUpdateInterval = 0.025;
  [motionManager startAccelerometerUpdates];
  
  
  // Game-Engine als Timer starten
  [NSTimer scheduledTimerWithTimeInterval:0.033 
                                   target:self 
                                 selector:@selector(gameEngine) 
                                 userInfo:nil
                                  repeats:YES];

}

-(void) gameEngine {
  if(gameMode == gameIsRunning) {
		
    /* GAMEENGINE  */

    // Scoreanzeige
    scorePlayer1.text = [NSString stringWithFormat:@"%d", scorePl1];
    scorePlayer2.text = [NSString stringWithFormat:@"%d", scorePl2];
    
    // Ballsteuerung
    ball.center = CGPointMake(ball.center.x + ballSpeedX,
                              ball.center.y + ballSpeedY);
    // Kollision mit Wand?
    int ballSize = ball.bounds.size.width;
    int viewPortWidth = self.view.bounds.size.width;
    int player1Width = player1.bounds.size.width;
    int player1Height = player1.bounds.size.height;
    
    if (ball.center.x + ballSize/2 > viewPortWidth ||
        ball.center.x - ballSize/2 < 0) {
      ballSpeedX = -ballSpeedX;
    }
    
    
    // an Plattform abprallen
    if (CGRectIntersectsRect(ball.frame, player1.frame)) {
      if (
          // von oben
          (ball.center.y + ballSize/2 >= player1.center.y - player1Height/2) &&
          // von links oben
          (ball.center.x + ballSize/2 >= player1.center.x - player1Width/2) && 
          // von rechts oben
          (ball.center.x - ballSize/2 <= player1.center.x + player1Width/2) &&
          // nur eine Schrittweite in den Schlaeger hinein
          (ball.center.y+ballSize/2 <= player1.center.y-player1Height/2+ballSpeedY) 
          ) {
        ballSpeedY = -ballSpeedY;
        ballSpeedX -= player1RichtungX * 0.5;
        //scorePl1++;
      }
    }
    
    // das Selbe noch einmal fuer den oberen Schlaeger:
    // (da Player1 und Player2 die selbe Breite und Hoehe haben
    // verwenden wir einfach die Masse von Player1 weiter)
    if (CGRectIntersectsRect(ball.frame, player2.frame)) {  
      if (
          // von unten
          (ball.center.y - ballSize/2 <= player2.center.y + player1Height/2) && 
          // von links unten
          (ball.center.x + ballSize/2 >= player2.center.x - player1Width/2) &&
          // von rechts unten
          (ball.center.x - ballSize/2 <= player2.center.x + player1Width/2) &&
          // nur eine Schrittweite in den Schlaeger hinein
          (ball.center.y-ballSize/2 >= player2.center.y+player1Height/2+ballSpeedY) 
          ) {
        ballSpeedY = -ballSpeedY;
        ballSpeedX -= player2RichtungX * 0.5;
        //scorePl1++;
      }
    }
    
    int viewPortHeight = self.view.bounds.size.height;

    // einfache KI
    /*if ( (ball.center.y < viewPortHeight/2) && (ballSpeedY < 0) ) {
      if (player2.center.x > ball.center.x) player2RichtungX=-4;
      if (player2.center.x < ball.center.x) player2RichtungX=4;
    }
    else player2RichtungX=0;*/

    // Ball im Aus?
    if (ball.center.y < 0) {
      scorePl1++;
      if (scorePl1<10) [self newRound]; else [self newGame];
    }
    if (ball.center.y > viewPortHeight) {
      scorePl2++;
      if (scorePl2<10) [self newRound]; else [self newGame];
    }


    // Sensorbewegung auslesen
    if (useMotionManager) {
      CMAccelerometerData *acceleration = motionManager.accelerometerData;
      player1RichtungX = 20*acceleration.acceleration.x;
      //player2RichtungX = -20*acceleration.acceleration.x;
    } 

    // Playerbewegung (Accelerometer)
    if ((player1.center.x + player1RichtungX > 0) &&
        (player1.center.x + player1RichtungX < viewPortWidth)) 
      player1.center = CGPointMake(player1.center.x + player1RichtungX,
                                   player1.center.y);
    
    if ((player2.center.x + player2RichtungX > 0) &&
        (player2.center.x + player2RichtungX < viewPortWidth))
      player2.center = CGPointMake(player2.center.x + player2RichtungX,
                                   player2.center.y);

    /* GAMEENGINE  ENDE */
		
  } else {
    
    /* START- BZW. PAUSENSCREEN  */
    
  }
}

- (void)newGame {
  scorePl1 = 0; // Score zuruecksetzen
  scorePl2 = 0; // Score zuruecksetzen
  ball.center = self.view.center; // Ball zurueck in die Mitte
  startScreen.hidden=NO;
  gameMode=gameIsPaused;
}

- (void)newRound {
	ball.center = self.view.center;
	ballSpeedX = random()%10 - 5;
}


- (void)touchesBegan:(NSSet *)touches withEvent:(UIEvent *)event {
	[self touchesMoved:touches withEvent:event];
}


- (void)touchesMoved:(NSSet *)touches withEvent:(UIEvent *)event {
  NSArray *allTouches = [touches allObjects]; 
  CGPoint coord = [[allTouches objectAtIndex:[allTouches count]-1] 
                   locationInView:self.view];
  
  if (gameMode==gameIsRunning) {
    if (!useMotionManager) {
      // Player 1
      if (coord.y > self.view.bounds.size.height/2) {
        if (coord.x>self.view.bounds.size.width/2) player1RichtungX=4;
        else player1RichtungX=-4;
      }
      // Player 2
      if (coord.y <= self.view.bounds.size.height/2) {
        if (coord.x > self.view.bounds.size.width/2) player2RichtungX=4;
        else player2RichtungX=-4;
      }
    }
  }	else {
    startScreen.hidden=YES;
    gameMode=gameIsRunning;
  }
}

- (void)touchesEnded:(NSSet *)touches withEvent:(UIEvent *)event {

  NSArray *allTouches = [touches allObjects]; 
  CGPoint coord = [[allTouches objectAtIndex:[allTouches count]-1] 
                   locationInView:self.view];
  
  if (gameMode==gameIsRunning) {
    if (!useMotionManager) {
      if (coord.y<=240) {
        player2RichtungX=0;
      }
      else {
        player1RichtungX=0;
      }
    }
  }
}


// unbenutzte Restfunktionen

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    // Return YES for supported orientations
  if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone) {
      return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
  } else {
      return YES;
  }
}

@end
